"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const webpack_libs_1 = require("./infra-comp-utils/webpack-libs");
const sls_libs_1 = require("./infra-comp-utils/sls-libs");
const configuration_lib_1 = require("./infra-comp-utils/configuration-lib");
const console_output_1 = require("./infra-comp-utils/console-output");
const clc = require('cli-color');
const infrastructure_components_1 = require("infrastructure-components");
/**
 *
 * @param configFilePath
 */
function build(configFilePath) {
    return __awaiter(this, void 0, void 0, function* () {
        // load and parse the configuration from the temporary folder
        // when building, we do not provide a stage - this should run the environments in build mode (considering all of them)
        const parsedConfig = yield configuration_lib_1.parseConfiguration(configFilePath, undefined, infrastructure_components_1.PARSER_MODES.MODE_BUILD);
        /*
        if (parsedConfig.stackType !== "SPA" ) {
            await runSlsCmd(`${SLS_PATH} plugin install --name serverless-dynamodb-local`);
        }*/
        yield sls_libs_1.createSlsYaml(parsedConfig.slsConfigs, true);
        exports.writeScriptsToPackageJson(configFilePath, parsedConfig.webpackConfigs.filter(wpConfig => wpConfig.name !== undefined).map(wpConfig => wpConfig.name), parsedConfig.environments, parsedConfig.supportOfflineStart, parsedConfig.supportCreateDomain);
        // now run the webpacks
        yield Promise.all(parsedConfig.webpackConfigs.map((wpConfig) => __awaiter(this, void 0, void 0, function* () {
            console.log("wpConfig: ", wpConfig);
            yield webpack_libs_1.runWebpack(wpConfig);
            console.log("--- done ---");
        })));
        console.log(`running ${parsedConfig.postBuilds.length} postscripts...`);
        // now run the post-build functions
        if (parsedConfig.stackType !== "SOA") {
            yield Promise.all(parsedConfig.postBuilds.map((postBuild) => __awaiter(this, void 0, void 0, function* () { return yield postBuild(); })));
        }
        else {
            yield Promise.all(parsedConfig.postBuilds.map((postBuild) => __awaiter(this, void 0, void 0, function* () { return yield postBuild({ serviceEndpoints: ["localhost:3001"] }); })));
        }
        yield require('infrastructure-components').fetchData("build", {
            stackname: parsedConfig.stackName
        });
        writeMessage();
    });
}
exports.build = build;
;
const writeMessage = () => {
    console.log(console_output_1.frameTop());
    console.log(console_output_1.emptyLine());
    console.log(console_output_1.frameText("Thank you for using Infrastructure-Components!", clc.magenta.bold));
    console.log(console_output_1.emptyLine());
    console.log(console_output_1.singleLine());
    console.log(console_output_1.emptyLine());
    console.log(console_output_1.frameText("The Infrastructure-Components are under Active Development", clc.green));
    console.log(console_output_1.emptyLine());
    console.log(console_output_1.frameText(" -- In case you need help or discovered a bug, please let us know:", clc.magenta));
    console.log(console_output_1.frameText("    https://spectrum.chat/infrastructure", clc.green));
    console.log(console_output_1.emptyLine());
    console.log(console_output_1.frameText(" -- Make sure you don't miss the next update or the next feature! ", clc.magenta));
    console.log(console_output_1.frameText("    - on Medium.com:  https://medium.com/@fzickert", clc.green));
    console.log(console_output_1.frameText("    - in our Docs:    https://infrastructure-components.readthedocs.io", clc.green));
    console.log(console_output_1.frameText("    - on our Website: https://www.infrastructure-components.com", clc.green));
    /*console.log(emptyLine());

    console.log(frameText(" -- You don't have an AWS account to deploy to? Try our managed service:", clc.magenta));
    console.log(frameText("    https://www.code-architect.com", clc.green));
    */
    console.log(console_output_1.emptyLine());
    console.log(console_output_1.frameBottom());
};
exports.writeScriptsToPackageJson = (configFilePath, webapps, environments, supportOfflineStart, supportCreateDomain) => {
    const fs = require('fs');
    try {
        fs.copyFileSync('package.json', 'package_last.json');
        let rawdata = fs.readFileSync('package.json');
        let packageJson = JSON.parse(rawdata);
        const domains = (supportCreateDomain == true || supportCreateDomain == undefined) ?
            environments.filter(env => env.domain !== undefined).map(env => {
                var result = {};
                result["domain-" + env.name] = `scripts .env domain ${configFilePath} ${env.name}`;
                return result;
            }) : [{}];
        packageJson["scripts"] = Object.assign({}, packageJson.scripts, ...environments.map(env => {
            var result = {};
            if (supportOfflineStart == true || supportOfflineStart == undefined) {
                result["start-" + env.name] = `scripts .env start ${configFilePath} ${env.name}`;
            }
            result["deploy-" + env.name] = `scripts .env deploy ${configFilePath} ${env.name}`;
            return result;
        }), ...webapps.map(app => {
            var obj = {};
            obj[app] = `scripts app ${configFilePath} ${app}`;
            return obj;
        }), ...domains);
        fs.writeFileSync('package.json', JSON.stringify(packageJson, null, 2));
    }
    catch (error) {
        console.error("Could not write scripts to package.json\n", error);
        return undefined;
    }
    ;
};
//# sourceMappingURL=build.js.map