"use strict";
/**
 * Starts an express-server on `localhost` that serves the client-web app on the port specified in
 * the environment variables or 3000 (default)
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const configuration_lib_1 = require("./infra-comp-utils/configuration-lib");
const sls_libs_1 = require("./infra-comp-utils/sls-libs");
const webpack_libs_1 = require("./infra-comp-utils/webpack-libs");
const console_output_1 = require("./infra-comp-utils/console-output");
const clc = require('cli-color');
const infrastructure_components_1 = require("infrastructure-components");
/**
 * uses the current serverless.yml (created by previous build!) to deploy the stack
 *
 * @param configFilePath
 */
function deploy(configFilePath, stage) {
    return __awaiter(this, void 0, void 0, function* () {
        const path = require('path');
        const rimraf = require("rimraf");
        // load and parse the configuration from the temporary folder
        const parsedConfig = yield configuration_lib_1.parseConfiguration(configFilePath, stage, infrastructure_components_1.PARSER_MODES.MODE_DEPLOY);
        // delete the build-folder
        rimraf.sync(parsedConfig.buildPath);
        // (re-)create the serverless.yml
        sls_libs_1.createSlsYaml(parsedConfig.slsConfigs, true);
        // now run the webpacks - except the web-targets
        yield Promise.all(parsedConfig.webpackConfigs.filter(wpConfig => wpConfig.target !== "web").map((wpConfig) => __awaiter(this, void 0, void 0, function* () {
            console.log("wpConfig: ", wpConfig);
            yield webpack_libs_1.runWebpack(wpConfig);
            console.log("--- server webpacks done ---");
        })));
        if (parsedConfig.stackType === "SPA") {
            // now run the web-targets webpacks - SPA-ONLY!
            yield Promise.all(parsedConfig.webpackConfigs.filter(wpConfig => wpConfig.target === "web").map((wpConfig) => __awaiter(this, void 0, void 0, function* () {
                console.log("wpConfig: ", wpConfig);
                yield webpack_libs_1.runWebpack(wpConfig);
                console.log("--- client webpacks done ---");
            })));
        }
        if (parsedConfig.stackType !== "SOA") {
            console.log(`running ${parsedConfig.postBuilds.length} postscripts...`);
            // now run the post-build functions
            yield Promise.all(parsedConfig.postBuilds.map((postBuild) => __awaiter(this, void 0, void 0, function* () { return yield postBuild(); })));
        }
        // start the sls-config
        yield sls_libs_1.deploySls(parsedConfig.stackName);
        const accountId = yield sls_libs_1.getAccountIDUsingAccessKey();
        //console.log("accountId: ", accountId);
        const staticBucketName = infrastructure_components_1.getStaticBucketName(accountId, parsedConfig.stackName, parsedConfig.assetsPath, stage);
        // we can now retrieve the endpoints
        var endpointMsg = undefined;
        var serviceEndpoints = undefined;
        if (parsedConfig.stackType === "SPA" || parsedConfig.stackType === "SOA") {
            yield require('infrastructure-components').fetchData("deploy", {
                stackname: parsedConfig.stackName,
                envi: stage,
                domain: parsedConfig.domain,
                endp: `http://${staticBucketName}.s3-website-${parsedConfig.region}.amazonaws.com`
            });
            endpointMsg = `http://${staticBucketName}.s3-website-${parsedConfig.region}.amazonaws.com`;
        }
        if (parsedConfig.stackType !== "SPA") {
            var endpointUrl = undefined;
            var eps = {};
            yield sls_libs_1.runSlsCmd("echo $(sls info)", data => {
                //console.log("data: " , data);
                eps = data.split(" ").reduce(({ inSection, endpoints }, val, idx) => {
                    //console.log("eval: " , val);
                    if (inSection && val.indexOf("https://") > -1) {
                        return { inSection: true, endpoints: endpoints.concat(val.indexOf("{proxy+}") == -1 ? [val] : []) };
                    }
                    if (val.startsWith("endpoints:")) {
                        return { inSection: true, endpoints: endpoints };
                    }
                    if (val.startsWith("functions:")) {
                        return { inSection: false, endpoints: endpoints };
                    }
                    return { inSection: inSection, endpoints: endpoints };
                }, { inSection: false, endpoints: [] });
                //console.log("endpoints" , eps)
            }, false);
            const data = Object.assign({
                stackname: parsedConfig.stackName,
                envi: stage,
                domain: parsedConfig.domain
            }, eps.endpoints.length > 0 ? { endp: eps.endpoints[0] } : {});
            yield require('infrastructure-components').fetchData("deploy", data);
            if (eps.endpoints.length > 0) {
                // the ServiceOrientedApp already has an endpoint, but it also has services
                if (endpointMsg !== undefined) {
                    serviceEndpoints = eps.endpoints;
                }
                else {
                    endpointMsg = eps.endpoints[0];
                }
            }
        }
        /// end of retrieving the endpoints!
        /* we can use the stage-arg here, because this is supposed to be the name of the environment anyway!
        const env = Array.isArray(parsedConfig.environments) && parsedConfig.environments.length > 0 ?
            parsedConfig.environments[0] : parsedConfig.environments;
    
        env !== undefined && env.name !== undefined ? env.name :*/
        if (parsedConfig.stackType !== "SPA") {
            // now run the web-targets webpacks - OTHER THAN SPA!
            yield Promise.all(parsedConfig.webpackConfigs.filter(wpConfig => wpConfig.target === "web").map((wpConfig) => __awaiter(this, void 0, void 0, function* () {
                console.log("wpConfig: ", wpConfig);
                yield webpack_libs_1.runWebpack(wpConfig);
                console.log("--- client webpacks done ---");
            })));
        }
        //for SOA, we need to provide the endoints into the build of the client,
        // we need to run the post-scripts after the web-target webpacks!
        if (parsedConfig.stackType === "SOA") {
            console.log(`SOA: running ${parsedConfig.postBuilds.length} postscripts...`);
            // now run the post-build functions
            yield Promise.all(parsedConfig.postBuilds.map((postBuild) => __awaiter(this, void 0, void 0, function* () { return yield postBuild({ serviceEndpoints: serviceEndpoints }); })));
        }
        // copy the client apps to the assets-folder
        console.log("start S3 Sync");
        yield Promise.all(
        // only copy webapps
        parsedConfig.webpackConfigs.filter(wpConfig => wpConfig.target === "web").map((wpConfig) => __awaiter(this, void 0, void 0, function* () {
            yield sls_libs_1.s3sync(parsedConfig.region, staticBucketName, path.join(parsedConfig.buildPath, wpConfig.name));
        })));
        if (endpointMsg !== undefined) {
            console.log(console_output_1.frameTop());
            console.log(console_output_1.emptyLine());
            console.log(console_output_1.frameText("Your deployment is complete!", clc.magenta.bold));
            console.log(console_output_1.emptyLine());
            console.log(console_output_1.frameText("Your React-App is now available at:", clc.magenta));
            console.log(console_output_1.frameText(endpointMsg, clc.green));
            console.log(console_output_1.emptyLine());
            if (serviceEndpoints !== undefined) {
                console.log(console_output_1.frameText("Your App has the following services:", clc.magenta));
                serviceEndpoints.forEach(endpoint => console.log(console_output_1.frameText(" - " + endpoint, clc.green)));
                console.log(console_output_1.emptyLine());
            }
            console.log(console_output_1.frameBottom());
        }
    });
}
exports.deploy = deploy;
;
//# sourceMappingURL=deploy.js.map