"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * returns the current working directory
 */
exports.currentAbsolutePath = () => process.cwd().toString().replace(/(?:\r\n|\r|\n)/g, "");
/**
 *
 * @param configFilePath
 * @return the absolute path to the main configuration of the project
 *
 */
exports.pathToConfigFile = (configFilePath) => {
    const path = require('path');
    return path.resolve(exports.currentAbsolutePath(), configFilePath.replace(/\.[^/.]+$/, ""));
};
/**
 * npm run build && node -r dotenv/config -e 'require(\"./src/config\").slsLogin()' dotenv_config_path=.dev.env && sls deploy && node -r dotenv/config -e 'require(\"./src/config\").s3sync()' dotenv_config_path=.dev.env",
 */
function loadEnv(env) {
    return __awaiter(this, void 0, void 0, function* () {
        if (env.endsWith(".env")) {
            const cmd = require('node-cmd');
            yield cmd.run(`export $(grep -v '^#' ${env} | xargs) `);
            const dotenv = require('dotenv');
            const result = dotenv.config({ path: env });
            if (result.error) {
                throw result.error;
            }
            //const { parsed: envs } = result;
            //console.log(envs);
            return true;
        }
        return false;
    });
}
exports.loadEnv = loadEnv;
;
function copyAssets(source, targetFolder) {
    const fs = require('fs');
    const path = require('path');
    var files = [];
    //check if folder needs to be created or integrated
    if (!fs.existsSync(targetFolder)) {
        fs.mkdirSync(targetFolder, { recursive: true });
    }
    //copy
    if (fs.lstatSync(source).isDirectory()) {
        files = fs.readdirSync(source);
        files.forEach(function (file) {
            var curSource = path.join(source, file);
            console.log("source: ", curSource);
            console.log("dest: ", path.join(targetFolder, path.parse(curSource).base));
            if (fs.lstatSync(curSource).isDirectory()) {
                copyFolderRecursiveSync(curSource, targetFolder);
            }
            else {
                fs.copyFileSync(curSource, path.join(targetFolder, path.parse(curSource).base));
                //fs.unlinkSync(curSource);
            }
        });
    }
}
exports.copyAssets = copyAssets;
function copyFolderRecursiveSync(source, target) {
    const fs = require('fs');
    const path = require('path');
    var files = [];
    //check if folder needs to be created or integrated
    var targetFolder = path.join(target, path.basename(source));
    if (!fs.existsSync(targetFolder)) {
        fs.mkdirSync(targetFolder);
    }
    //copy
    if (fs.lstatSync(source).isDirectory()) {
        files = fs.readdirSync(source);
        files.forEach(function (file) {
            var curSource = path.join(source, file);
            if (fs.lstatSync(curSource).isDirectory()) {
                copyFolderRecursiveSync(curSource, targetFolder);
            }
            else {
                fs.copyFileSync(curSource, path.join(targetFolder, path.parse(curSource).base));
            }
        });
    }
}
exports.copyFolderRecursiveSync = copyFolderRecursiveSync;
//# sourceMappingURL=system-libs.js.map