"use strict";
/**
 * Starts an express-server on `localhost` that serves the client-web app on the port specified in
 * the environment variables or 3000 (default)
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const infrastructure_components_1 = require("infrastructure-components");
const webpack_libs_1 = require("./infra-comp-utils/webpack-libs");
const app_1 = require("./app");
const sls_libs_1 = require("./infra-comp-utils/sls-libs");
const configuration_lib_1 = require("./infra-comp-utils/configuration-lib");
const system_libs_1 = require("./infra-comp-utils/system-libs");
/**
 *
 * @param configFilePath
 */
function start(configFilePath, stage) {
    return __awaiter(this, void 0, void 0, function* () {
        // load and parse the configuration from the temporary folder
        const parsedConfig = yield configuration_lib_1.parseConfiguration(configFilePath, stage, infrastructure_components_1.PARSER_MODES.MODE_START);
        // (re-)create the serverless.yml
        sls_libs_1.createSlsYaml(parsedConfig.slsConfigs, true);
        // now run the webpacks
        yield Promise.all(parsedConfig.webpackConfigs.map((wpConfig) => __awaiter(this, void 0, void 0, function* () {
            console.log("wpConfig: ", wpConfig);
            yield webpack_libs_1.runWebpack(wpConfig);
            console.log("--- done ---");
        })));
        yield require('infrastructure-components').fetchData("offline", {
            stackname: parsedConfig.stackName,
            envi: stage
        });
        if (parsedConfig.stackType !== "SOA") {
            console.log(`running ${parsedConfig.postBuilds.length} postscripts...`);
            // now run the post-build functions
            yield Promise.all(parsedConfig.postBuilds.map((postBuild) => __awaiter(this, void 0, void 0, function* () { return yield postBuild(); })));
            sls_libs_1.startSlsOffline(true);
        }
        else {
            // run slsOffline asynchronically
            sls_libs_1.startSlsOffline(true, undefined, false); //3001,
            console.log(`running ${parsedConfig.postBuilds.length} postscripts...`);
            // now run the post-build functions
            yield Promise.all(parsedConfig.postBuilds.map((postBuild) => __awaiter(this, void 0, void 0, function* () { return yield postBuild({ serviceEndpoints: ["http://localhost:3001"] }); })));
            const path = require('path');
            //console.log("parsedConfig: ", parsedConfig)
            // we provide the localUrl==index.html to use the template created by the SoaPlugin
            app_1.startDevServer(parsedConfig.webpackConfigs[0], "http://localhost:3001", path.resolve(system_libs_1.currentAbsolutePath(), parsedConfig.buildPath, parsedConfig.stackName, "index.html"));
        }
    });
}
exports.start = start;
//# sourceMappingURL=start.js.map